@@ -236,103 +236,109 @@ static int bcm2712_iommu_attach_dev(struct iommu_domain *domain, struct device *
 	return -EINVAL;
 }
 
-static void bcm2712_iommu_detach_dev(struct iommu_domain *domain, struct device *dev)
-{
-	(void)domain;
-	(void)dev;
-}
-
 static int bcm2712_iommu_map(struct iommu_domain *domain, unsigned long iova,
-			     phys_addr_t pa, size_t bytes, int prot, gfp_t gfp)
+			     phys_addr_t pa, size_t bytes, size_t count,
+			     int prot, gfp_t gfp, size_t *mapped)
 {
 	struct bcm2712_iommu *mmu = domain_to_mmu(domain);
+	u32 entry = MMMU_PTE_VALID | (pa >> MMU_PAGE_SHIFT);
+	u32 align = (u32)(iova | pa | bytes);
+	unsigned int p;
 
-	(void)gfp;
-	iova -= mmu->dma_iova_offset;
-	if (iova >= APERTURE_BASE && iova + bytes <= APERTURE_TOP) {
-		unsigned int p;
-		u32 entry = MMMU_PTE_VALID | (pa >> MMU_PAGE_SHIFT);
-		u32 align = (u32)(iova | pa | bytes);
-
-		/* large page and write enable flags */
-		if (!(align & ((1 << HUGEPAGE_SHIFT) - 1)))
-			entry |= FIELD_PREP(MMMU_PTE_PAGESIZE_MASK, 3);
-		else if (!(align & mmu->superpage_mask) && mmu->superpage_mask)
-			entry |= FIELD_PREP(MMMU_PTE_PAGESIZE_MASK, 2);
-		else if (!(align &  mmu->bigpage_mask) && mmu->bigpage_mask)
-			entry |= FIELD_PREP(MMMU_PTE_PAGESIZE_MASK, 1);
-		if (prot & IOMMU_WRITE)
-			entry |= MMMU_PTE_WRITEABLE;
-
-		/* Ensure tables are cache-coherent with CPU */
-		if (!mmu->dirty) {
-			dma_sync_sgtable_for_cpu(mmu->dev, mmu->sgt, DMA_TO_DEVICE);
-			mmu->dirty = true;
-		}
-
-		iova -= APERTURE_BASE;
-		for (p = iova >> MMU_PAGE_SHIFT;
-		     p < (iova + bytes) >> MMU_PAGE_SHIFT; p++) {
-			mmu->nmapped_pages += !(mmu->tables[p]);
-			mmu->tables[p] = entry++;
-		}
-	} else if (iova + bytes > APERTURE_BASE || iova != pa) {
-		dev_warn(mmu->dev, "%s: iova=0x%lx pa=0x%llx size=0x%llx OUT OF RANGE!\n",
+	/* Reject if at least the first page is not within our aperture */
+	if (iova < mmu->dma_iova_offset + APERTURE_BASE ||
+	    iova + bytes > mmu->dma_iova_offset + APERTURE_TOP) {
+		dev_warn(mmu->dev, "%s: iova=0x%lx pa=0x%llx bytes=0x%lx OUT OF RANGE\n",
 			 __func__, iova,
-			 (unsigned long long)pa, (unsigned long long)bytes);
+			 (unsigned long long)pa, (unsigned long)bytes);
+		*mapped = 0;
+
 		return -EINVAL;
 	}
 
+	/* large page and write enable flags */
+	if (!(align & ((1 << HUGEPAGE_SHIFT) - 1)))
+		entry |= FIELD_PREP(MMMU_PTE_PAGESIZE_MASK, 3);
+	else if (!(align & mmu->superpage_mask) && mmu->superpage_mask)
+		entry |= FIELD_PREP(MMMU_PTE_PAGESIZE_MASK, 2);
+	else if (!(align &  mmu->bigpage_mask) && mmu->bigpage_mask)
+		entry |= FIELD_PREP(MMMU_PTE_PAGESIZE_MASK, 1);
+	if (prot & IOMMU_WRITE)
+		entry |= MMMU_PTE_WRITEABLE;
+
+	/* Ensure tables are cache-coherent with CPU */
+	if (!mmu->dirty) {
+		dma_sync_sgtable_for_cpu(mmu->dev, mmu->sgt, DMA_TO_DEVICE);
+		mmu->dirty = true;
+	}
+
+	/* Make iova relative to table base; amalgamate count pages */
+	iova -= (mmu->dma_iova_offset + APERTURE_BASE);
+	bytes = min(APERTURE_SIZE - iova, count * bytes);
+
+	/* Iterate over table by smallest native IOMMU page size */
+	for (p = iova >> MMU_PAGE_SHIFT;
+	     p < (iova + bytes) >> MMU_PAGE_SHIFT; p++) {
+		mmu->nmapped_pages += !(mmu->tables[p]);
+		mmu->tables[p] = entry++;
+	}
+
+	*mapped = bytes;
+
 	return 0;
 }
 
 static size_t bcm2712_iommu_unmap(struct iommu_domain *domain, unsigned long iova,
-				  size_t bytes, struct iommu_iotlb_gather *gather)
+				  size_t bytes, size_t count,
+				  struct iommu_iotlb_gather *gather)
 {
 	struct bcm2712_iommu *mmu = domain_to_mmu(domain);
+	unsigned int p;
 
-	if (iova >= mmu->dma_iova_offset + APERTURE_BASE &&
-	    iova + bytes <= mmu->dma_iova_offset + APERTURE_TOP) {
-		unsigned int p;
+	if (iova < mmu->dma_iova_offset + APERTURE_BASE ||
+	    iova + bytes > mmu->dma_iova_offset + APERTURE_TOP)
+		return 0;
 
-		/* Record just the lower and upper bounds in "gather" */
-		if (gather) {
-			bool empty = (gather->end <= gather->start);
+	/* Record just the lower and upper bounds in "gather" */
+	if (gather) {
+		bool empty = (gather->end <= gather->start);
 
-			if (empty || gather->start < iova)
-				gather->start = iova;
-			if (empty || gather->end < iova + bytes)
-				gather->end = iova + bytes;
-		}
+		if (empty || gather->start < iova)
+			gather->start = iova;
+		if (empty || gather->end < iova + bytes)
+			gather->end = iova + bytes;
+	}
 
-		/* Ensure tables are cache-coherent with CPU */
-		if (!mmu->dirty) {
-			dma_sync_sgtable_for_cpu(mmu->dev, mmu->sgt, DMA_TO_DEVICE);
-			mmu->dirty = true;
-		}
+	/* Ensure tables are cache-coherent with CPU */
+	if (!mmu->dirty) {
+		dma_sync_sgtable_for_cpu(mmu->dev, mmu->sgt, DMA_TO_DEVICE);
+		mmu->dirty = true;
+	}
 
-		/* Clear table entries, this marks the addresses as illegal */
-		iova -= (mmu->dma_iova_offset + APERTURE_BASE);
-		for (p = iova >> MMU_PAGE_SHIFT;
-		     p < (iova + bytes) >> MMU_PAGE_SHIFT;
-		     p++) {
-			mmu->nmapped_pages -= !!(mmu->tables[p]);
-			mmu->tables[p] = 0;
-		}
+	/* Make iova relative to table base; amalgamate count pages */
+	iova -= (mmu->dma_iova_offset + APERTURE_BASE);
+	bytes = min(APERTURE_SIZE - iova, count * bytes);
+
+	/* Clear table entries, this marks the addresses as illegal */
+	for (p = iova >> MMU_PAGE_SHIFT;
+	     p < (iova + bytes) >> MMU_PAGE_SHIFT;
+	     p++) {
+		mmu->nmapped_pages -= !!(mmu->tables[p]);
+		mmu->tables[p] = 0;
 	}
 
 	return bytes;
 }
 
-static void bcm2712_iommu_sync_range(struct iommu_domain *domain,
-				     unsigned long iova, size_t size)
+static int bcm2712_iommu_sync_range(struct iommu_domain *domain,
+				    unsigned long iova, size_t size)
 {
 	struct bcm2712_iommu *mmu = domain_to_mmu(domain);
 	unsigned long iova_end;
 	unsigned int i, p4;
 
 	if (!mmu || !mmu->dirty)
-		return;
+		return 0;
 
 	/* Ensure tables are cleaned from CPU cache or write-buffer */
 	dma_sync_sgtable_for_device(mmu->dev, mmu->sgt, DMA_TO_DEVICE);
@@ -377,6 +383,8 @@ static void bcm2712_iommu_sync_range(struct iommu_domain *domain,
 			}
 		}
 	}
+
+	return 0;
 }
 
 static void bcm2712_iommu_sync(struct iommu_domain *domain,
@@ -418,9 +426,8 @@ static void bcm2712_iommu_domain_free(struct iommu_domain *domain)
 
 static const struct iommu_domain_ops bcm2712_iommu_domain_ops = {
 	.attach_dev	 = bcm2712_iommu_attach_dev,
-	.detach_dev	 = bcm2712_iommu_detach_dev,
-	.map		 = bcm2712_iommu_map,
-	.unmap		 = bcm2712_iommu_unmap,
+	.map_pages	 = bcm2712_iommu_map,
+	.unmap_pages	 = bcm2712_iommu_unmap,
 	.iotlb_sync      = bcm2712_iommu_sync,
 	.iotlb_sync_map  = bcm2712_iommu_sync_range,
 	.flush_iotlb_all = bcm2712_iommu_sync_all,
@@ -501,7 +508,7 @@ static struct iommu_group *bcm2712_iommu_device_group(struct device *dev)
 }
 
 static int bcm2712_iommu_of_xlate(struct device *dev,
-				  struct of_phandle_args *args)
+				  const struct of_phandle_args *args)
 {
 	struct platform_device *iommu_dev;
 	struct bcm2712_iommu *mmu;
@@ -640,7 +647,7 @@ done_err:
 	return ret;
 }
 
-static int bcm2712_iommu_remove(struct platform_device *pdev)
+static void bcm2712_iommu_remove(struct platform_device *pdev)
 {
 	struct bcm2712_iommu *mmu = platform_get_drvdata(pdev);
 
@@ -649,8 +656,6 @@ static int bcm2712_iommu_remove(struct platform_device *pdev)
 	if (mmu->sgt)
 		dma_free_noncontiguous(&pdev->dev, TABLES_ALLOC_SIZE,
 				       mmu->sgt, DMA_TO_DEVICE);
-
-	return 0;
 }
 
 static const struct of_device_id bcm2712_iommu_of_match[] = {
